package vivadaylight3.interlock.blocks;

import java.util.HashMap;
import java.util.List;
import java.util.Random;
import java.util.Set;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.client.renderer.texture.IconRegister;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.Icon;
import net.minecraft.util.MathHelper;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.World;
import universalelectricity.core.block.IConductor;
import universalelectricity.core.block.IConnectionProvider;
import universalelectricity.core.electricity.ElectricityPack;
import universalelectricity.core.electricity.IElectricityNetwork;
import universalelectricity.prefab.block.BlockAdvanced;
import vivadaylight3.interlock.Interlock;
import vivadaylight3.interlock.tileentity.TileEntityEnergiserBlock;

/**
 * Energiser block class
 * @author VivaDaylight3
 *
 */

public class EnergiserBlock extends BlockAdvanced implements IElectricityNetwork{
	
	private Icon blockIcon;
	private Icon iconTop;
	private Icon iconInput;
	private Icon iconOutput;
	private Icon iconSide;
	
	public static final int ENERGISER_BLOCK_METADATA = 0;
	
	public static final int WATTS_PER_TICK = 1;
	
	private TileEntityEnergiserBlock tileEntity = new TileEntityEnergiserBlock();

	public EnergiserBlock(int par1, Material par2Material) {
		super(par1, par2Material);
		setCreativeTab(Interlock.tabInterlock);
		setUnlocalizedName("energiserBlock");
		setHardness(1.0F);
		setStepSound(Block.soundMetalFootstep);
	}
	
	public void registerIcons(IconRegister iconRegister){
    	
		blockIcon = iconRegister.registerIcon("Interlock:energiserBlock");
		iconTop = iconRegister.registerIcon("Interlock:energiserBlock_top");
		iconInput = iconRegister.registerIcon("Interlock:energiserBlock_input");
		iconOutput = iconRegister.registerIcon("Interlock:energiserBlock_output");
		iconSide = iconRegister.registerIcon("Interlock:energiserBlock_side");
        
    }
	
	@Override
	public Icon getIcon(int side, int metadata)
	{
		String which = Interlock.getBlockSide(side, metadata, ENERGISER_BLOCK_METADATA);
		
		if (which == "top"){
			
			return iconTop;
			
		}else if (which == "main"){
			
			return blockIcon;
			
		}else if (which == "input"){
			
			return iconInput;
			
		}else if (which == "output"){
			
			return iconOutput;
			
		}else{
			
			return iconSide;
			
		}
		
	}
	
	@Override
	public boolean isOpaqueCube(){
		
		return false;
		
	}
	
	@Override
	public boolean renderAsNormalBlock(){
		
		return false;
		
	}
	
	public void onBlockPlacedBy(World world, int x, int y, int z, EntityLiving entityLiving, ItemStack itemStack){
		
		int metadata = world.getBlockMetadata(x, y, z);
		int angle = MathHelper.floor_double((entityLiving.rotationYaw * 4.0F / 360.0F) + 0.5D) & 3;
		int change = 0;
		
		switch(angle){
		
			case 0:
				change = 0;
				break;
			
			case 1:
				change = 1;
				break;
				
			case 2:
				change = 2;
				break;
				
			case 3:
				change = 3;
				break;
		
		}
		
		world.setBlockMetadataWithNotify(x, y, z, ENERGISER_BLOCK_METADATA + change, 3);
				
	}
		
	public boolean onUseWrench(World par1World, int x, int y, int z, EntityPlayer par5EntityPlayer, int side, float hitX, float hitY, float hitZ){
		
		int metadata = par1World.getBlockMetadata(x, y, z);
		int change = 0;
				
		switch (metadata)
		{
			case 4:
				change = 1;
				break;
			case 5:
				change = 2;
				break;
			case 6:
				change = 3;
				break;
			case 7:
				change = 0;
				break;
		}
		
		par1World.setBlockMetadataWithNotify(x, y, z, ENERGISER_BLOCK_METADATA + change, 3);
		
		return true;
		
	}
	
	@Override
	public boolean onMachineActivated(World par1World, int x, int y, int z, EntityPlayer par5EntityPlayer, int side, float hitX, float hitY, float hitZ){
				
		if(!par1World.isRemote){
			
			TileEntity tileEntity = par1World.getBlockTileEntity(x, y, z);

	        if (tileEntity == null || par5EntityPlayer.isSneaking()){
	        	
	        	return false;
	        	        	
	        }
			
	        par5EntityPlayer.openGui(Interlock.instance, 3, par1World, x, y, z);
	        	        
	        return true;
		}
		
		return true;
		
	}
	
	public ItemStack getEnergiserBlock(){
		
		return new ItemStack(this.blockID, 1, ENERGISER_BLOCK_METADATA);
		
	}
	
	@Override
	public ItemStack getPickBlock(MovingObjectPosition target, World world, int x, int y, int z)
	{
		int id = idPicked(world, x, y, z);

		if (id == 0)
		{
			return null;
		}

		Item item = Item.itemsList[id];

		if (item == null)
		{
			return null;
		}

		int metadata = getDamageValue(world, x, y, z);

		return new ItemStack(id, 1, metadata);
	}
	
	public int idDropped(int par1, Random random, int zero){
    	
    	return Interlock.energiserBlock.blockID;
    	
    }
    
    public int quantityDropped(Random par1Random)
    {
        return 1;
    }
    
    public TileEntity createNewTileEntity(World par1World)
    {
        return new TileEntityEnergiserBlock();
    }

	@Override
	public void startProducing(TileEntity tileEntity,
			ElectricityPack electricityPack) {
		
	}

	@Override
	public void startProducing(TileEntity tileEntity, double amperes,
			double voltage) {
		
	}

	@Override
	public boolean isProducing(TileEntity tileEntity) {
		return false;
	}

	@Override
	public void stopProducing(TileEntity tileEntity) {
		
	}

	@Override
	public void startRequesting(TileEntity tileEntity,
			ElectricityPack electricityPack) {
		
	}

	@Override
	public boolean isRequesting(TileEntity tileEntity) {
		return true;
	}

	@Override
	public void stopRequesting(TileEntity tileEntity) {
		
	}

	@Override
	public ElectricityPack getProduced(TileEntity... ignoreTiles) {
		return null;
	}

	@Override
	public ElectricityPack getRequest(TileEntity... ignoreTiles) {
		return null;
	}

	@Override
	public ElectricityPack getRequestWithoutReduction() {
		return null;
	}

	@Override
	public ElectricityPack consumeElectricity(TileEntity tileEntity) {
		return null;
	}

	@Override
	public HashMap<TileEntity, ElectricityPack> getProducers() {
		return null;
	}

	@Override
	public List<TileEntity> getProviders() {
		return null;
	}

	@Override
	public HashMap<TileEntity, ElectricityPack> getConsumers() {
		return null;
	}

	@Override
	public List<TileEntity> getReceivers() {
		return null;
	}

	@Override
	public Set<IConductor> getConductors() {
		return null;
	}

	@Override
	public double getTotalResistance() {
		return 0;
	}

	@Override
	public double getLowestCurrentCapacity() {
		return 0;
	}

	@Override
	public void cleanUpConductors() {
		
	}

	@Override
	public void refreshConductors() {
		
	}

	@Override
	public void mergeConnection(IElectricityNetwork network) {
		
	}

	@Override
	public void splitNetwork(IConnectionProvider splitPoint) {
		
	}

	@Override
	public void startRequesting(TileEntity tileEntity, double amperes,
			double voltage) {
		
	}

}
