package vivadaylight3.interlock.tileentity;

import java.util.EnumSet;
import java.util.HashSet;
import java.util.Set;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.network.INetworkManager;
import net.minecraft.network.packet.Packet;
import net.minecraft.network.packet.Packet250CustomPayload;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import universalelectricity.core.block.IElectricityStorage;
import universalelectricity.core.electricity.ElectricityNetwork;
import universalelectricity.core.electricity.ElectricityNetworkHelper;
import universalelectricity.core.electricity.ElectricityPack;
import universalelectricity.core.electricity.IElectricityNetwork;
import universalelectricity.core.item.ElectricItemHelper;
import universalelectricity.core.item.IItemElectric;
import universalelectricity.core.vector.Vector3;
import universalelectricity.core.vector.VectorHelper;
import universalelectricity.prefab.network.IPacketReceiver;
import universalelectricity.prefab.network.PacketManager;
import universalelectricity.prefab.tile.TileEntityElectricityRunnable;
import universalelectricity.prefab.tile.TileEntityElectricityStorage;
import vivadaylight3.interlock.Interlock;
import vivadaylight3.interlock.blocks.EnergiserBlock;

import com.google.common.io.ByteArrayDataInput;

import cpw.mods.fml.common.network.PacketDispatcher;
import cpw.mods.fml.common.network.Player;

public class TileEntityEnergiserBlock extends TileEntityElectricityStorage implements IElectricityStorage, IPacketReceiver, ISidedInventory{
		
	private ItemStack[] containingItems = new ItemStack[2];
	
	public final Set<EntityPlayer> playersUsing = new HashSet<EntityPlayer>();
	
	public ForgeDirection outputDirection;
	private ForgeDirection inputDirection;
	
	public IElectricityNetwork outputNetwork;
	private IElectricityNetwork inputNetwork;
	
	public TileEntity outputTile;
	private TileEntity inputTile;
	private TileEntityFenceBlock fenceBlock;
		
	public final double JOULES_MAX = 1000000.0;
	
	private boolean isRequesting;

	private double joulesStored;
	
	public void setDirections(){
		
		this.outputDirection = ForgeDirection.getOrientation(Interlock.getBlockOutput(this.getBlockMetadata(), EnergiserBlock.ENERGISER_BLOCK_METADATA));
		this.inputDirection = ForgeDirection.getOrientation(Interlock.getBlockInput(this.getBlockMetadata(), EnergiserBlock.ENERGISER_BLOCK_METADATA));
		
		this.outputTile = VectorHelper.getConnectorFromSide(worldObj, new Vector3(this), this.outputDirection);
		this.inputTile = VectorHelper.getConnectorFromSide(worldObj, new Vector3(this), this.inputDirection);
	
		this.outputNetwork = ElectricityNetworkHelper.getNetworkFromTileEntity(this.outputTile, this.outputDirection);
		this.inputNetwork = ElectricityNetworkHelper.getNetworkFromTileEntity(this.inputTile, this.inputDirection);
	
	}
	
	@Override
	public void updateEntity()
	{
		super.updateEntity();

		if (!this.isDisabled())
		{
			if (!this.worldObj.isRemote)
			{

				/**
				 * Decharge electric item.
				 */
				
				System.out.println("Stored Joules: "+this.getJoules());
				
				this.setJoules(this.getJoules() + ElectricItemHelper.dechargeItem(this.containingItems[1], this.getMaxJoules() - this.getJoules(), this.getVoltage()));
				
				this.setDirections();
				
				if (this.outputNetwork != null && this.inputNetwork != this.outputNetwork)
				{
					if(this.getJoules() < getMaxJoules()){
						
						double inputWatts = Math.min(this.inputNetwork.getProduced(this).getWatts(), Math.min(this.getJoules(), JOULES_MAX));
						
						this.isRequesting = true;
						
						this.inputNetwork.startRequesting(this, inputWatts / this.getVoltage(), this.getVoltage());
						
					}else{
						
						this.inputNetwork.stopRequesting(this);
						
					}
					
					double outputWatts = Math.min(this.outputNetwork.getRequest(this).getWatts(), Math.min(this.getJoules(), JOULES_MAX));

					if (this.getJoules() > 0 && outputWatts > 0)
					{
												
						boolean consumersAreFences = this.outputNetwork.getConsumers().containsKey(this.fenceBlock);
						
						if(consumersAreFences){
							
							this.outputNetwork.startProducing(this, outputWatts / this.getVoltage(), this.getVoltage());
							this.setJoules(this.getJoules() - outputWatts);
							
						}
					}
					else
					{
						this.outputNetwork.stopProducing(this);
					}
				}
			}
		}
				
		if (!this.worldObj.isRemote)
		{
			if (this.ticks % 3 == 0)
			{
				for (EntityPlayer player : this.playersUsing)
				{
					PacketDispatcher.sendPacketToPlayer(getDescriptionPacket(), (Player) player);
				}
			}
		}

	}
	
	@Override
	public Packet getDescriptionPacket()
	{
		return PacketManager.getPacket(Interlock.CHANNEL, this, this.getJoules(), this.disabledTicks);
	}

	@Override
	public void handlePacketData(INetworkManager network, int type, Packet250CustomPayload packet, EntityPlayer player, ByteArrayDataInput dataStream)
	{
		try
		{
			this.setJoules(dataStream.readDouble());
			this.disabledTicks = dataStream.readInt();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public boolean canConnect(ForgeDirection direction)
	{		
		this.setDirections();
		
		if(direction == this.outputDirection){
			
			return true;
			
		}else if(direction == this.inputDirection){
			
			return true;
			
		}else{
		
			return false;
		
		}
	}

	@Override
	protected EnumSet<ForgeDirection> getConsumingSides()
	{
		return EnumSet.allOf(ForgeDirection.class);
	}
	
	@Override
	public ElectricityPack getRequest()
	{
		if (this.isRequesting)
		{
			return new ElectricityPack(EnergiserBlock.WATTS_PER_TICK / this.getVoltage(), this.getVoltage());
		}
		else
		{
			return new ElectricityPack();
		}
	}

	@Override
	public double getMaxJoules() {
		return JOULES_MAX;
	}
	
	@Override
	public void readFromNBT(NBTTagCompound nbt)
	{
		super.readFromNBT(nbt);
		
		joulesStored = nbt.getDouble("electricityStored");
		
		NBTTagList var2 = nbt.getTagList("Items");
		this.containingItems = new ItemStack[this.getSizeInventory()];

		for (int var3 = 0; var3 < var2.tagCount(); ++var3)
		{
			NBTTagCompound var4 = (NBTTagCompound) var2.tagAt(var3);
			byte var5 = var4.getByte("Slot");

			if (var5 >= 0 && var5 < this.containingItems.length)
			{
				this.containingItems[var5] = ItemStack.loadItemStackFromNBT(var4);
			}
		}
	}
	
	@Override
	public void writeToNBT(NBTTagCompound nbt)
	{
		super.writeToNBT(nbt);
		
		NBTTagCompound nbt2 = new NBTTagCompound();
		
		nbt.setDouble("electricityStored", joulesStored);
		
		NBTTagList var2 = new NBTTagList();
		
		var2.appendTag(nbt2);

		for (int var3 = 0; var3 < this.containingItems.length; ++var3)
		{
			if (this.containingItems[var3] != null)
			{
				NBTTagCompound var4 = new NBTTagCompound();
				var4.setByte("Slot", (byte) var3);
				this.containingItems[var3].writeToNBT(var4);
				var2.appendTag(var4);
			}
		}

		nbt.setTag("Items", var2);
	}
	
	@Override
	public int getSizeInventory()
	{
		return this.containingItems.length;
	}

	@Override
	public ItemStack getStackInSlot(int par1)
	{
		return this.containingItems[par1];
	}
	
	@Override
	public ItemStack getStackInSlotOnClosing(int par1)
	{
		if (this.containingItems[par1] != null)
		{
			ItemStack var2 = this.containingItems[par1];
			this.containingItems[par1] = null;
			return var2;
		}
		else
		{
			return null;
		}
	}
	
	@Override
	public void setInventorySlotContents(int par1, ItemStack par2ItemStack)
	{
		this.containingItems[par1] = par2ItemStack;

		if (par2ItemStack != null && par2ItemStack.stackSize > this.getInventoryStackLimit())
		{
			par2ItemStack.stackSize = this.getInventoryStackLimit();
		}
	}
	
	@Override
	public String getInvName()
	{
		return "Energiser";
	}
	
	@Override
	public boolean isInvNameLocalized()
	{
		return false;
	}
	
	@Override
	public int getInventoryStackLimit()
	{
		return 1;
	}
	
	@Override
	public boolean isStackValidForSlot(int slotID, ItemStack itemstack)
	{
		return itemstack.getItem() instanceof IItemElectric;
	}
	
	@Override
	public int[] getAccessibleSlotsFromSide(int slotID)
	{
		return new int[] { 0, 1 };
	}
	
	@Override
	public boolean canInsertItem(int slotID, ItemStack itemstack, int side)
	{
		if (this.isStackValidForSlot(slotID, itemstack))
		{

			if (slotID == 1){
				
				return ((IItemElectric) itemstack.getItem()).getProvideRequest(itemstack).getWatts() > 0;
			}
		}
		return false;
	}
	
	@Override
	public boolean canExtractItem(int slotID, ItemStack itemstack, int side)
	{
		if (this.isStackValidForSlot(slotID, itemstack))
		{
			if (slotID == 1)
			{
				return ((IItemElectric) itemstack.getItem()).getProvideRequest(itemstack).getWatts() <= 0;
			}
		}

		return false;

	}
	
	@Override
	public boolean isUseableByPlayer(EntityPlayer par1EntityPlayer)
	{
		return this.worldObj.getBlockTileEntity(this.xCoord, this.yCoord, this.zCoord) != this ? false : par1EntityPlayer.getDistanceSq(this.xCoord + 0.5D, this.yCoord + 0.5D, this.zCoord + 0.5D) <= 64.0D;
	}

	@Override
	public ItemStack decrStackSize(int i, int j) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void openChest() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void closeChest() {
		// TODO Auto-generated method stub
		
	}
	
}
